#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <math.h>
#include <float.h>
#include <sys/param.h>
#include "ppm.h"

char *progname;

void usage(void) {
  fprintf(stderr, "usage: %s [-h] [-o file] file\n", progname);
  fprintf(stderr, "\t-h\t\tprint usage\n"); 
  fprintf(stderr, "\t-o file\t\toutput image file\n");
  exit(0);
}

int main (int argc, char *argv[]) {
  FILE *in = stdin;
  FILE *out = stdout;

  char signature_name[MAXPATHLEN];
  char input_name[MAXPATHLEN] = "(stdin)";
  char output_name[MAXPATHLEN] = "(stdout)";

  int rows, cols, format;
  pixval maxval;
  int row, col;
  pixel **image;
  unsigned char *tmp;

  int i, c, n;
  int shift;

  progname = argv[0];

#ifdef __EMX__
  _fsetmode(in, "b");
  _fsetmode(out, "b");
#endif
  
  ppm_init(&argc, argv);
  
  while ((c = getopt(argc, argv, "h?o:")) != EOF) {
   switch (c) {
      case 'h':
      case '?':
        usage();
        break; 
      case 'o':
        if ((out = fopen(optarg, "wb")) == NULL) {
          fprintf(stderr, "%s: unable to open output file %s\n", progname, optarg);
          exit(1);
        }
        strcpy(output_name, optarg);
        break; 
    }
  }

  argc -= optind; 
  argv += optind;
          
  if (argc > 1) { 
    usage();
    exit(1);
  }

  if (argc == 1 && *argv[0] != '-')
    if ((in = fopen(argv[0], "rb")) == NULL) {
      fprintf(stderr, "%s: unable to open input file %s\n", progname, argv[0]);
      exit(1);
    }
    else  
      strcpy(input_name, argv[0]);

  ppm_readppminit(in, &cols, &rows, &maxval, &format);
  image = ppm_allocarray(cols, rows);
  for (row = 0; row < rows; row++)
    ppm_readppmrow(in, image[row], cols, maxval, format);
  fclose(in);

  shift = 0;
  while (maxval > 255) shift++, maxval >= 1;

  tmp = malloc(cols * 3 * sizeof(char));
  for (row = 0; row < rows; row++) {
    unsigned char *t = tmp;
    for (col = 0; col < cols; col++) {
      *t++ = image[row][col].r >> shift;
      *t++ = image[row][col].g >> shift;
      *t++ = image[row][col].b >> shift;
    }
    fwrite(tmp, sizeof(char), cols * 3, out);
  }
  fclose(out);
        
  ppm_freearray(image, rows);
    
  exit(0);
}
